/*
 * @Author: your name
 * @Date: 2021-10-14 17:17:53
 * @LastEditTime: 2022-05-03 16:59:33
 * @LastEditors: your name
 * @Description: In User Settings Edit
 * @FilePath: \Foshan4A\src\router\index.js
 */
import Vue from "vue";
import Router from "vue-router";
import store from "@/store";

Vue.use(Router);

/* Layout */
// import Layout from '@/layout'
import routesOne from "./routes/routes-file-one";
import routesTwo from "./routes/routes-file-two";
import routesThree from "./routes/routes-file-three";
import routesFileFour from "./routes/routes-file-four";
import routesFive from "./routes/routes-file-five";
/**
 * Note: sub-menu only appear when route children.length >= 1
 * Detail see: https://panjiachen.github.io/vue-element-admin-site/guide/essentials/router-and-nav.html
 *
 * hidden: true                   if set true, item will not show in the sidebar(default is false)
 * alwaysShow: true               if set true, will always show the root menu
 *                                if not set alwaysShow, when item has more than one children route,
 *                                it will becomes nested mode, otherwise not show the root menu
 * redirect: noRedirect           if set noRedirect will no redirect in the breadcrumb
 * name:'router-name'             the name is used by <keep-alive> (must set!!!)
 * meta : {
    roles: ['admin','editor']    control the page roles (you can set multiple roles)
    title: 'title'               the name show in sidebar and breadcrumb (recommend set)
    icon: 'svg-name'/'el-icon-x' the icon show in the sidebar
    breadcrumb: false            if set false, the item will hidden in breadcrumb(default is true)
    activeMenu: '/example/list'  if set path, the sidebar will highlight the path you set
  }
 */

const routesPush = (arr, routes) => {
  if (!Array.isArray(arr)) return "请传入数组格式路由表";
  for (let i = 0; i < arr.length; i++) {
    routes.unshift(arr[i]);
  }
};

/**
 * constantRoutes
 * a base page that does not have permission requirements
 * all roles can be accessed
 */
export const constantRoutes = [
  {
    path: "/login",
    component: () => import("@/views/login/index"),
    hidden: true,
  },
  {
    path: "/404",
    component: () => import("@/views/404"),
    hidden: true,
  },
  // { path: '/', redirect: '/nopower', component: () => import('@/views/noPower'), hidden: true }
  // 404 page must be placed at the end !!!
];

// 动态路由
export const asyncRoutes = [];
// 插入路由
routesPush(
  [
    ...routesOne,
    ...routesTwo,
    ...routesThree,
    ...routesFileFour,
    ...routesFive
  ],
  asyncRoutes
);
// asyncRoutes.push({ path: '/', component: () => import('@/views/noPower'), hidden: true })
asyncRoutes.push({
  path: "*",
  component: () => import("@/views/404"),
  hidden: true,
});

const createRouter = () =>
  new Router({
    // mode: 'history', // require service support
    scrollBehavior: () => ({ y: 0 }),
    routes: constantRoutes,
  });

const router = createRouter();

router.beforeEach((to, from, next) => {
  // 如果 要 from(离开) 的页面是 keepAlive缓存的，
  // 再根据 deepth 来判断是前进还是后退
  if (from.meta.keepAlive) {
    if (to.meta.deepth && to.meta.deepth > from.meta.deepth) {
      store.dispatch("keepAlive/editPage", from);
    } else {
      store.dispatch("keepAlive/deletePage", to);
    }
  }
  // 如果要to(进入)的页面是需要keepAlive缓存的，把name push进keepAlive数组中
  if (to.meta.keepAlive) {
    const currentPage = store.state.keepAlive.keepAlivePages.find(
      (page) => page.name === to.name
    );
    if (currentPage) {
      if (currentPage.fullPath !== to.fullPath) {
        next(currentPage.fullPath);
      }
    } else {
      store.dispatch("keepAlive/addPage", to);
    }
  }
  next();
});

// Detail see: https://github.com/vuejs/vue-router/issues/1234#issuecomment-357941465
export function resetRouter () {
  const newRouter = createRouter();
  router.matcher = newRouter.matcher; // reset router
}

export default router;
